function results = QS_se_no(Y,mh,symb,info)
% PURPOSE: computes Q(m) statistic for spatial association of qualitative
% variables
% ---------------------------------------------------
%  USAGE: results = QS_se(Y,mh,symb,info)
%  where: Y  = 1*n vector of qualitative variables entered as 1,2,...,K
%         mh = n*m matrix with the m-surrounding of each spatial observation
%         symb = (k^m)*m matrix containing symbols
%         info = an (optional) structure variable with input options
%         info.plot  = 'yes' / ['no'] flag for plotting symbols histogram
%         info.print = 'yes' / ['no'] print the result
% ---------------------------------------------------
%  RETURNS: results.qs - the value of the statistic (non-equiprobable case)
%           results.gs - the value of the statistic (equiprobable case)
%           results.ns  - vector for producing bar plot with frequency of symbols
%           results.nusi- number of symbols
%  --------------------------------------------------
%  SEE ALSO: m_hist0, bar
% ---------------------------------------------------
% REFERENCES: Ruiz M, Lopez F, Paez A (2009) Testing for Spatial
% Association of Qualitative Data Using Symbolic Dynamics, Journal of
% Geographical Systems
% ---------------------------------------------------

% written by:
% Fernando A. Lpez Hernndez, 2009
% Facultad de C.C. de la Empresa
% Dpto. Metodos Cuantitativos e Informticos
% Universidad Politcnica de Cartagena
% E-mail: Fernando.Lopez@upct.es
% &
% Antonio Paez, 2009
% School of Geography and Earth Sciences
% McMaster University
% E-mail: paezha@mcmaster.ca

[n1 m]=size(mh);%n1: number of symbolized locations, m: size of m-surrounding
[nusi n2]=size(symb);%nusi: number of symbols, 
lns=zeros(1,nusi);
S=zeros(1,n1);
N=length(Y);
%if length(Y)~=n1, error('Error: mh and Y are different lengths'), end 
%if m~=n2, error('Error: my and symb are different lengths'), end 
if nargin<4
    info.plot='no';
else
    info.plot=lower(info.plot);
end
if nargin<4
    info.print='no';
else
    info.print=lower(info.print);
end
%% Calculate the probabilities of each of K qualitative classes.
Ymax=max(Y);
pm=zeros(1,Ymax);
for k=1:Ymax
    pm(k)=sum(Y==k)/N;
end
%% Evaluate the probabilities of each symbol under the null.
for k=1:nusi
    q(k)=1;
    for j=1:m
    q(k)=q(k)*pm(symb(k,j));%^sum(symb(k,:)==j)
    end
end
%% Expected frequency of symbols under the null
sf0=n1*q;%The probability of the symbol under the null times the number of symbolized locations
%%
Z=Y(mh);
for i=1:n1
    for k=1:nusi
        if isequal(symb(k,:),Z(i,:))==1, S(i)=k; end
    end
end
%Empirical frequency of symbols
results.ns=hist(S',1:nusi);
if strcmp(info.plot,'yes')
    figure;
    %plot barsempirical frequency of symbols
    bar(results.ns/n1,'w');hold
    set(gca,'xlim',[0,nusi+1]);
end
for i=1:nusi
    if results.ns(i)==0, lns(i)=0; else lns(i)=log(results.ns(i)/n1); end
end
hm=-sum((results.ns/n1).*lns);
hm0=sum((results.ns/n1).*log(q));
qs=-2*n1*sum(hm0+hm);
gs=2*n1*(log(nusi)-hm);
%es95 = chi2inv(0.95,nusi-1);
[chi2inv(0.95,nusi-1) gs];
results.qs=qs;%Results for non-equiprobable values: Nominal classes do not appear with same frequency
results.gs=gs;%Results for equiprobable values Each nominal class appears with same frequency
%results.S=S';
results.nusi=nusi;
if strcmp(info.print,'yes')
    fprintf(1,'\n')
    fprintf(1,'  Sample size               (N) = %9.0f\n',N)
    fprintf(1,'  Symbolized observations   (R) = %9.0f\n',n1)
    fprintf(1,'  Number of classes         (k) = %9.0f\n',Ymax)
    fprintf(1,'  Size of m-surrounding     (m) = %9.0f\n',m)
    fprintf(1,'  Degree of overlap         (r) = %9.0f\n',m-(N-m)/(n1-1))
    fprintf(1,'  Number of symbols         (n) = %9.0f\n',nusi)
    fprintf(1,'  Ratio R/n                     = %9.2f\n',n1/nusi)
    fprintf(1,'  5k^m                          = %9.2f\n',5*Ymax^m)
    fprintf(1,'  Frequency of categories       = ');fprintf(1,'%9.4f ',pm');fprintf(1,'\n');
    fprintf(1,'\n')
    fprintf(1,'Test QS for spatial dependence in qualitative data\n');
    fprintf(1,'        Test                      Value     DF    p-value \n');
    fprintf(1,'   QS (equiprobab.)  =          %9.2f',gs);fprintf(1,'    %2.0f',nusi-1);fprintf(1,' %9.4f \n',1-chi2cdf(gs,nusi-1));
    fprintf(1,'   QS (non-equiprob.)=          %9.2f',qs);fprintf(1,'    %2.0f',nusi-1);fprintf(1,' %9.4f \n',1-chi2cdf(qs,nusi-1));
end